# -*- coding: utf-8 -*-
"""Domain Driven Design framework."""
from .nginx_base_exception import NginxBaseException


class NginxError(NginxBaseException):

    """Generic exception for Nginx root"""
    def __init__(self, msg="Nginx root error", original_exception=None):
        super(NginxError, self).__init__(msg + (": %s" % original_exception))
        self.original_exception = original_exception



# -*- coding: utf-8 -*-
"""Domain Driven Design framework."""
import os
from collections import namedtuple
from jinja2 import Environment, FileSystemLoader

TEMPLATE_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), "template/apigw")
Variable = namedtuple('Variable', ['key', 'value'])


class Block:

    def __init__(self):
        self.sets = []

    def set_var(self, key, value):
        self.sets.append(Variable(key, value))

    def load(self):
        pass

    def dump(self, template_name, render_key_pairs):
        j2_env = Environment(loader=FileSystemLoader(TEMPLATE_DIR),
                             trim_blocks=True)
        return j2_env.get_template(template_name).render(render_key_pairs)

    def append(self, block):
        raise NotImplementedError

    def remove(self):
        pass



# -*- coding: utf-8 -*-
"""Domain Driven Design framework."""
from collections import namedtuple
from ddd_base import ValueObject
from .block import Block

ReverseProxyStrategy = namedtuple('ReverseProxyStrategy', ['strategy', 'rule'])


class Location(ValueObject, Block):

    def __init__(self, name=None, scope=None, proxy=None):
        super().__init__()
        self.name = name
        self.scope = scope
        self.proxy = proxy

    def __eq__(self, other):
        if not isinstance(other, Location):
            return NotImplemented

        return self.scope == other.scope \
            and self.proxy == other.proxy \
            and self.name == other.name

    def append(self, block):
        pass



# -*- coding: utf-8 -*-
"""Domain Driven Design framework."""
from ddd_base import ValueObject
from collections import namedtuple
from .block import Block

MapDefinition = namedtuple('MapDefinition', ['key', 'value'])
MapKeyParis = namedtuple('MapKeyParis', ['key', 'value'])


class Map(ValueObject, Block):

    def __init__(self, definition=None):
        super().__init__()
        self.definition = definition
        self.key_pairs = []

    def __eq__(self, other):
        if not isinstance(other, Map):
            return NotImplemented

        return self.definition.key == self.definition.key \
            and self.definition.value == other.definition.value \
            and self.key_pairs == other.key_pairs

    def append(self, block):
        self.key_pairs.append(block)



# -*- coding: utf-8 -*-
"""Domain Driven Design framework."""
from ddd_base import Entity
from .block import Block


class Upstream(Entity, Block):

    def __init__(self, name):
        super().__init__()
        self.name = name
        self.servers = []

    def append(self, block):
        self.servers.append(block)



"""Setuptools entry point."""
import codecs
import os
from setuptools import find_packages

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


HERE = os.path.abspath(os.path.dirname(__file__))
ABOUT = {}
with open(os.path.join(HERE, 'ddd_nginx', '__version__.py')) as f:
    exec(f.read(), ABOUT)

with open(os.path.join(HERE, 'requirements', 'base.txt')) as req:
    REQUIREMENTS = [line.strip() for line in req.readlines() if line.strip() and not line.strip().startswith('#')]

CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Natural Language :: English',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Topic :: Software Development :: Libraries :: Python Modules'
]

dirname = os.path.dirname(__file__)

long_description = (
    codecs.open(os.path.join(dirname, 'README.rst'), encoding='utf-8').read() + '\n' +
    codecs.open(os.path.join(dirname, 'CHANGES.rst'), encoding='utf-8').read()
)

setup(
    name='ddd-nginx',
    version=ABOUT['__version__'],
    description='DDD nginx framework for python',
    long_description=long_description,
    author='Sun Wei',
    author_email='wayde.sun@gmail.com',
    url='https://github.com/sunwei/ddd-nginx',
    packages=find_packages(exclude=['tests*']),
    install_requires=REQUIREMENTS,
    classifiers=CLASSIFIERS)



# -*- coding: utf-8 -*-
"""Domain Driven Design framework."""
from ddd_base import Aggregate
from .block import Block
from .server import Server
from .upstream import Upstream
from .map import Map
from .exception import NginxError


class Nginx(Aggregate, Block):

    def __init__(self, host):
        super().__init__()
        self._namespace = None
        self.host = host
        self.upstreams = []
        self.maps = []
        self.servers = []

    @property
    def namespace(self):
        return self._namespace

    @namespace.setter
    def namespace(self, value):
        self._namespace = value + '.' + self.host

    def dumps(self, path):
        pass

    def append(self, block):
        if isinstance(block, Server):
            self.servers.append(block)
        elif isinstance(block, Map):
            self.maps.append(block)
        elif isinstance(block, Upstream):
            self.upstreams.append(block)
        else:
            raise NginxError()
